require 'spec_helper'

generate_tester_classes 'JSON', :JSON

describe SerialBox::Serializers::JSON do
  context "[basic schema]" do
    it "should serialize using #serialize with a method" do
      obj = JSONSerializeMethodTester.new('value')
      JSON.parse(obj.to_json, :create_additions => false).should eql({'json_class' => 'JSONSerializeMethodTester', 'var' => 'value'})

      obj = JSON.parse('{"json_class":"JSONSerializeMethodTester","var":"value"}')
      obj.var.should eql('value')
    end

    it "should serialize using #serialize with an instance variable" do
      obj = JSONSerializeIvarTester.new('value')
      JSON.parse(obj.to_json, :create_additions => false).should eql({'json_class' => 'JSONSerializeIvarTester', 'var' => 'value'})

      obj = JSON.parse('{"json_class":"JSONSerializeIvarTester","var":"value"}')
      obj.instance_variable_get(:@var).should eql('value')
    end

    it "should serialize into a custom JSON key" do
      obj = JSONSerializeIntoTester.new('value')
      JSON.parse(obj.to_json, :create_additions => false).should eql({'json_class' => 'JSONSerializeIntoTester', 'custom' => 'value'})

      obj = JSON.parse('{"json_class":"JSONSerializeIntoTester","custom":"value"}')
      obj.var.should eql('value')
    end

    it "should serialize using #serializer and #deserializer with object methods" do
      obj = JSONSerializerMethodTester.new('value')
      JSON.parse(obj.to_json, :create_additions => false).should eql({'json_class' => 'JSONSerializerMethodTester', 'var_json' => 'value'})

      obj = JSON.parse('{"json_class":"JSONSerializerMethodTester","var_json":"value"}')
      obj.var_getter.should eql('value')
    end

    it "should serialize using #serializer and #deserializer with blocks" do
      obj = JSONSerializerBlockTester.new('value')
      JSON.parse(obj.to_json, :create_additions => false).should eql({'json_class' => 'JSONSerializerBlockTester', 'var_json' => 'value'})

      obj = JSON.parse('{"json_class":"JSONSerializerBlockTester","var_json":"value"}')
      obj.instance_variable_get(:@var).should eql('value')
    end
  end

  context "[sub-objects]" do
    it "should correctly serialize and deserialize sub-objects" do
      subobj = JSONSerializeMethodTester.new('value')
      obj    = JSONSerializeMethodTester.new(subobj)
      JSON.parse(obj.to_json, :create_additions => false).should eql({
                                                                         'json_class' => 'JSONSerializeMethodTester',
                                                                         'var'        => {'json_class' => 'JSONSerializeMethodTester',
                                                                                          'var'        => 'value'}})

      obj = JSON.parse('{"json_class":"JSONSerializeMethodTester","var":{"json_class":"JSONSerializeMethodTester","var":"value"}}')
      obj.var.should be_kind_of(JSONSerializeMethodTester)
      obj.var.var.should eql('value')
    end

    it "should correctly serialize and deserialize special objects in generic objects" do
      obj    = [JSONSerializeMethodTester.new('value')]
      JSON.parse(obj.to_json, :create_additions => false).should eql([{'json_class' => 'JSONSerializeMethodTester',
                                                                       'var'        => 'value'}])

      obj = JSON.parse('[{"json_class":"JSONSerializeMethodTester","var":"value"}]')
      obj.should be_kind_of(Array)
      obj.first.should be_kind_of(JSONSerializeMethodTester)
      obj.first.var.should eql('value')
    end
  end
end
